/*
 *  net_msg.h	 Defines & structs for msgs to QNX 4.0+ net
 *
 *  Copyright (C) by Quantum Software Systems 1990.  All rights reserved.
 */
#ifndef __NET_MSG_H_INCLUDED

#ifndef __TYPES_H_INCLUDED
#include <sys/types.h>
#endif

#pragma pack(1);

/***************************************************************************
 *
 *	RUNTIME ERRORS
 *
 *  The following are runtime errors for the 4.10 qnx network manager
 */
/*
 *  nerr_event() increments this error code when an invalid code is passed
 */
#define NET_RERR_BAD_ERRCODE	    0

/*
 *	tx_prep(): should not have see a _VC_PLS_GROW_VID queue pkt type
 */
#define NET_RERR_TXPREP_EPGVID      1

/*
 *	tx_prep(): should not have see a _VC_PLS_GROW_LCL queue pkt type
 */
#define NET_RERR_TXPREP_EPGLCL      2

/*
 *  tx_prep(): process manager enqueued a vid poll NACK back to a node
 *  that sent us a vid poll packet on min 1, max 3 vid(s).  Apparently,
 *  we have "forgotten" about these vid(s) (did we reboot recently?), and
 *  the remote nid thinks it still has valid vid(s) to us, which it will
 *  tear down as soon as it receives our transmitted NACK.
 */
#define NET_RERR_TXPREP_POLLNACK    3

/*
 *	tx_prep(): unknown queue packet type
 */
#define NET_RERR_TXPREP_EQPKT       4

/* 
 *  local_qpkt(): bad status came back from the process manager after net
 *  dequeued a _VC_PLS_GROW_VID request, then sent a vc_attach_lreq
 *  to the process manager
 */
#define NET_RERR_LQPKT_GROWVID      5

/*
 *  local_qpkt(): bad status came back from the process manager after net
 *  dequeued a _VC_PLS_GROW_LCL request, then sent a vc_attach_lreq
 *  to the process manager
 */
#define NET_RERR_LQPKT_GROWLCL      6     

/*
 *  tx_error(): as a result of a failed tx of a vc_attach ctrl pkt,
 *	sent a vc_attached with bad status to proc  
 */
#define NET_RERR_TXERR_EVCAT        7     

/*
 *  tx_error(): as a result of a failed tx of a vc_attached ctrl pkt,
 *	sent a vc_detach with bad status to proc  
 */
#define NET_RERR_TXERR_EVCATD       8     

/*
 *  tx_error(): failed tx of a vc_detach.  No action taken
 */
#define NET_RERR_TXERR_EVCDT        9     

/*
 *  tx_error(): failed tx of a send on a vc.  Local process freed w/EIO
 */
#define NET_RERR_TXERR_ESEND        10   

/*
 *  tx_error(): failed tx of a reply on a vc.  No action taken.
 */
#define NET_RERR_TXERR_EREPLY       11   

/*
 *  tx_error(): failed tx of raw data.  No action taken.
 */
#define NET_RERR_TXERR_ERAW         12   

/*
 *  tx_error(): should never see a _VC_PLS_GROW_VID queue pkt here.
 *	No action taken.
 */
#define NET_RERR_TXERR_EPGV         13   

/*
 *  tx_error(): should never see a _VC_PLS_GROW_LCL queue pkt here.
 *	No action taken.
 */
#define NET_RERR_TXERR_EPGL         14   

/*
 *  tx_error(): failed tx of a _VC_SIGNAL_PROP.  No action taken.
 */
#define NET_RERR_TXERR_ESIGPRP      15   

/*
 *  tx_error(): failed tx of a _VC_POLL.  No action taken.
 */
#define NET_RERR_TXERR_EPOLL        16   

/*
 *  tx_error(): failed tx of a _VC_POLL_ACK. No action taken.
 */
#define NET_RERR_TXERR_EPOLLA       17   

/*
 *  tx_error(): failed tx of a _VC_POLL_NACK. No action taken.
 */
#define NET_RERR_TXERR_EPOLLN       18

/*
 *  tx_error(): failed tx of a _VC_DRVR_ONLY. No action taken.
 */
#define NET_RERR_TXERR_EDONLY       19

/*
 *  tx_error(): failed tx of a _VC_SIGNAL_BLK.  Local process freed w/EIO
 */
#define NET_RERR_TXERR_ESIGBLK      20   

/*
 *  tx_error(): failed tx of a _VC_FORCE_READY. No action taken.
 */
#define NET_RERR_TXERR_EFRDY        21   

/*
 *  tx_error(): unknown queue packt type.  No action taken.
 */
#define NET_RERR_TXERR_EQPTYPE      22   

/*
 *  handle_rx(): bad status back from the process manager when
 *  _VC_ATTACH msg was sent to him
 */
#define NET_RERR_HRX_VCAT           23

/*
 *  handle_rx(): bad status back from the process manager when
 *  _VC_ATTACHED msg was sent to him
 */
#define NET_RERR_HRX_VCAD           24 

/*
 *  handle_rx(): bad status back from the process manager when
 *  _VC_DETACH msg was sent to him
 */
#define NET_RERR_HRX_VCDT           25

/*
 *  handle_rx(): bad status back from the process manager when
 *  _VC_SIGNAL_PROP (_VC_AT_RREQ) msg was sent to him
 */
#define NET_RERR_HRX_SPRP           26

/*
 *  handle_rx(): we received a vid poll NACK back from a remote node
 *  that we previously must have tx'd a vid poll pkt to, and the remote node
 *  has "forgotten" about the 1->3 vids.  Anyways, we will send msg to
 *  the process manager, and he will tear down these vids.
 */
#define NET_RERR_HRX_POLLNACK       27

/*
 *  handle_rx(): after receiving a poll/poll_ack/poll_nack pkt in
 *  from the net, we sent the appropriate poll msg to the process manager
 *  and he replied back with a bad status
 */
#define NET_RERR_HRX_POLLMSG        28

/*
 *  handle_rx(): encountered unknown control packet type
 */
#define NET_RERR_HRX_CTRL           29

/*
 *  handle_rx(): encountered unknown data packet type
 */
#define NET_RERR_HRX_DATA           30

/*
 *  handle_rx(): encountered unknown packet type
 */
#define NET_RERR_HRX_UNKNOWN        31

/*
 *  set_txerr_time(): internal queue pkt status error
 */
#define NET_RERR_STT_EQPSTAT        32

/*
 *  check_queues(): failed tx: no more drivers
 */
#define NET_RERR_CKQ_NMRDRVR        33

/*
 *  check_queues(): after failed tx, nodemap element gone
 */
#define NET_RERR_CKQ_NMGONE         34

/*
 *  check_queues(): nodemap miss - logical nid not in table
 */
#define NET_RERR_CKQ_NMAPMIS        35

/*
 *  check_queues(): no driver after successful nodemap lookup
 */
#define NET_RERR_CKQ_ENMDRST        36

/*
 *  handle_raw_rx(): unknown raw packet received
 */
#define NET_RERR_RRX_UNKNWN         37

/*
 *  raw_tx_nq(): logical address of zero illegally specified
 */
#define NET_RERR_RTN_LANZERO        38

/*
 *  raw_tx_nq(): logical address not found in driver table
 */
#define NET_RERR_RTN_LANNFND        39

/*
 *  raw_tx_dn(): invalid raw slot on completion of raw tx 
 */
#define NET_RERR_RTD_RAWSLOT        40

/*
 *  check_queues(): tx of raw data failed
 */
#define NET_RERR_RTD_RAWTXER        41

/*
 *  stuff_phys(): lookup of lan address in nodemap element failed
 */
#define NET_RERR_STP_NMLANA         42

/*
 *  set_txerr_time(): lookup of lan address in nodemap element failed
 */
#define NET_RERR_STT_NMLANA         43

/*
 *  handle_rx(): possible duplicate vid sequence number
 */
#define NET_RERR_RX_PDUPVSEQ        44

/*
 *  drvr_best(): driver has gone away
 */
#define NET_RERR_DRB_DRVR           45

/*
 *  drvr_best(): nodemap card value has changed
 */
#define NET_RERR_DRB_NMCARD         46

/*
 *  nm_load(): unable to alloc nodemap element
 */
#define NET_RERR_NML_ALLOC          47

/*
 *  get_tx_vseq(): vid was zero
 */
#define NET_RERR_IVS_VZERO          48

/*
 *  reset_vseq(): vid was zero
 */
#define NET_RERR_RVS_VZERO          49

/*
 *  pdup_vseq(): vid was zero
 */
#define NET_RERR_DVS_VZERO          50

/*
 *  set_txerr_time(): lookup of nodemap element failed
 */
#define NET_RERR_STT_NMLU           51

/* 52 is unused - used to be NET_RERR_NML_NOSLOT */

/*
 *  nm_free(): nodemap element not in hashed linked list
 */
#define NET_RERR_NMF_NLST  		    53

/*
 *  raw_tx_nq(): invalid raw slot at start of raw tx 
 */
#define NET_RERR_RTN_RAWSLOT        54

/*
 *  handle_raw_rx(): app could not buffer raw pkt (null pointer returned)
 */
#define NET_RERR_RRX_APPNOBUF		55

/*
 *	29 june 93: added the following to net re: netmap info changing
 *
 *	56 - driver regs
 *	57 - auto-learn from rxd packet
 *	58 - from netmap msg
 */

/*
 *  get_tx_vseq(): vid was too large, even after masking
 */
#define NET_RERR_IVS_V2BIG          60

/*
 *  reset_vseq(): vid was too large, even after masking
 */
#define NET_RERR_RVS_V2BIG          61

/*
 *  check_queues(): cannot find a bridge node to this logical nid
 */
#define NET_RERR_CKQ_BRGMIS         62

/*
 *  brg_buf_free(): illegal data_len
 */
#define NET_RERR_BBF_DLEN           63

/*
 *  brg_relay_pkt(): unable to allocate bridge buffer packet of this length
 */
#define NET_RERR_BRP_BLEN           64

/*
 *  brg_relay_pkt(): unable to allocate queue packet
 */
#define NET_RERR_BRP_QPKT           65

/*
 *  brg_relay_pkt(): nodemap lookup of (brgd) dst_nid failed
 */
#define NET_RERR_BRP_NML            66

/*
 *  brg_relay_pkt(): can't talk to directly, and no bridge, either
 */
#define NET_RERR_BRP_BNN            67

/*
 *  brg_relay_pkt(): no driver - drvr_best() failed
 */
#define NET_RERR_BRP_DB             68

/*
 *  brg_relay_pkt(): alloc of pkt bufs segs failed
 */
#define NET_RERR_BRP_ALC            69


/****************************************************************************/

#ifdef __386__
#define _NET_DRVR_REG		 0x418		/* 32-bit net driver reg & dreg msgs*/
#define _NET_DRVR_DREG		 0x419
#else
#define _NET_DRVR_REG		 0x410		/* 16-bit net driver reg & dreg msgs*/
#define _NET_DRVR_DREG		 0x411
#endif

#define _NET_WNODEMAP		 0x420		/* netmap util uses to load net maps*/
#define _NET_RNODEMAP		 0x421		/* netmap util uses to read net maps*/

#define _NET_RNDCNTS 		 0x42C		/* get node/driver xaction counts	*/

#define _NET_INFO    		 0x430		/* netinfo uses to get error log	*/
#define _NET_DRVR_INFO 		 0x438		/* netboot uses - drvr hw_types  	*/


#define _NET_RAW_REG32		 0x448		/* 32-bit raw app reg + dreg		*/
#define _NET_RAW_DREG32		 0x449
#define _NET_RAW_REG16		 0x440		/* 16-bit raw app reg + dreg		*/
#define _NET_RAW_DREG16		 0x441		/* process deregisters from raw rx	*/

#ifdef __386__                          /* avoid src code changes in raw apps*/
#define _NET_RAW_REG		 _NET_RAW_REG32
#define _NET_RAW_DREG		 _NET_RAW_DREG32
#else
#define _NET_RAW_REG		 _NET_RAW_REG16
#define _NET_RAW_DREG		 _NET_RAW_DREG16
#endif

#define _NET_INFO_DRVR 		 0x450		/* ask each driver process for info	*/

#define _NET_RBRGTBL  		 0x460		/* netmap util uses to read brg tbl */


#define _32_BIT_NET   		 0x001		/* drivers and raw apps, in subtype */

/* FYI: driver reg & dreg structs are in net_drvr.h */

struct _net_wnodemap {
	short unsigned int		type;
	short unsigned int		subtype;
	char					text[260];
};

struct _net_rnodemap {
	short unsigned int		type;
	short unsigned int		subtype;
	nid_t					log_nid;
	short unsigned int		zero1;
	short unsigned int		zero2;
};

struct _card_info {
	short unsigned int		lan_addr;
	unsigned char			phys_addr[6];
	time_t					last_tx_fail;
};

struct _net_rnodemap_reply {
	short unsigned int		status;
	short unsigned int		zero1;
	short unsigned int		num_card;
	short unsigned int		zero2;
	struct _card_info		card[];
};


struct _net_rndcnts {
	short unsigned int		type;
	short unsigned int		subtype;
	short unsigned int		zero1;
	short unsigned int		zero2;
};

struct _net_rndcnts_reply {
	short unsigned int		status;
	short unsigned int		zero1;
	short unsigned int		num_nodes;
	short unsigned int		num_drvrs;
	long					cnts[];
};

struct _net_rbrgtbl {
	short unsigned int		type;
	short unsigned int		subtype;
	short unsigned int		zero1;
	short unsigned int		zero2;
};

struct _brg_tbl {
	short unsigned int		 lan_addr;
	short unsigned int		 logical_addr;
};

struct _net_rbrgtbl_reply {
	short unsigned int		status;
	short unsigned int		zero1;
	short unsigned int		num_brgs;
	short unsigned int		zero2;
	struct _brg_tbl  		brg[];
};

struct _net_drvr_info {
	short unsigned int		type;
	short unsigned int		subtype;
	short unsigned int		zero1;
	short unsigned int		zero2;
};

struct _net_drvr_info_struct {
	short unsigned int		lan_addr;
	short unsigned int		zero1;
	short unsigned int		hw_type;
	short unsigned int		hw_subtype;
	short unsigned int		pid;
	short unsigned int		zero2;
};

struct _net_drvr_info_reply {
	short unsigned int				status;
	short unsigned int				zero1;
	short unsigned int				num_drvrs;
	short unsigned int				zero2;
	struct _net_drvr_info_struct	di[];
};

#define	_NET_NI		0		/* nerr_event err_type: no info */
#define	_NET_LN		1		/* nerr_event err_type: logical nid */
#define	_NET_PN		2		/* nerr_event err_type: physical nid */
#define	_NET_ST		3		/* nerr_event err_type: errno status */

struct _nerr_event {
    long				    time;
    short unsigned int		lan_addr;
    short unsigned int		err_code;
    short unsigned int		err_type;
	union {
		nid_t				log_nid;
		char				phys_nid[6];	/* should be IEEE thingie */
		short unsigned int	status;
	} un ;
};

struct _net_info {
	short unsigned int		type;
	short unsigned int		subtype;
	short unsigned int		zero1;
	short unsigned int		zero2;
	short unsigned int		start_index;
	short unsigned int		zero3;
	short unsigned int		num_room4;
	short unsigned int		zero4;
};

struct _net_info_reply {
	short unsigned int		status;
	short unsigned int		zero1;
	short unsigned int		zero2;
	short unsigned int		zero3;
	short unsigned int		num_here;
	short unsigned int		zero4;
	short unsigned int		more;
	short unsigned int		zero5;
	struct _nerr_event		list[];
};

#define _RPT_LEN	4				/* max len of rx protocol type */
#define	_ANY_RLA	0xFFFF			/* wildcard for rx_lan_addr	   */

struct _net_raw_reg {
    short int		type;
    short int		subtype;
	short int		rpt_len;
	short int		zero1;
    unsigned char	rpt[_RPT_LEN];	/* rx protocol type	*/
    short int		rx_lan_addr;	/* specific or wildcard */
	short int		zero2;
	short unsigned	rx_alloc_fn_seg;/* segs must all be same */
	long			rx_alloc_fn_ofs;
	short unsigned	rx_done_fn_seg;
	long			rx_done_fn_ofs;
	short unsigned	tx_done_fn_seg;
	long			tx_done_fn_ofs;
    short unsigned	ds;				/* ds relative to task's LDT	*/
    short int		zero3;
	short unsigned	txd_q_seg;		/* must be ds above */		
	long			txd_q_ofs;
    long			zero4;
    long			zero5;
};

struct _net_raw_reg_reply {
    short int		status;
    short unsigned	handle;			/* for deregister msg			*/
    short unsigned	cs;				/* seg mapped into net's LDT	*/
	short unsigned	zero1;
    short unsigned	ds;				/* seg mapped into net's LDT	*/
	short unsigned	zero2;
    short unsigned	free_q;			/* for tx						*/
	short unsigned	zero3;
    short unsigned	work_q;			/* for tx						*/
	short unsigned	zero4;
    short unsigned	net_qseg;		/* for tx						*/
	short unsigned	zero5;
    short unsigned	net_mid;		/* for tx						*/
	short unsigned	zero6;
};

/*
 *  After a net_register(), when raw data of the specified protocol
 *	type arrives, Net will far-call the app's alloc_fn() with [ax] set to
 *	the length of the raw data.  ds will be set to the aliased ds value
 *	in Net's LDT.  alloc_fn() will return in [ax dx] a far pointer 
 *	to the allocated buffer, zero means discard - unable to alloc a 
 *	buffer - the data will be discarded.
 *
 *	Net will then have the driver copy the data directly from the
 *	network card to the app's raw buffer.
 *
 *	Net will then far call the done_fn() with [ax] set to the logical 
 *  network address, and [dx bx cx] set to the source physical node ID 
 *
 *  done_fn() will return a value in [ax] which is a proxy for Net to
 *  trigger, zero means to not trigger anyone
 *
 */

struct _net_raw_dreg {
    short int		type;
    short unsigned	handle;
};

struct _net_raw_dreg_reply {
    short int		status;
};


/****************************************************************************/

#pragma pack();

#define __NET_MSG_H_INCLUDED
#endif
