
/*
	filename:	net_drvr.h
	purpose:	defines & structs specific to the 4.10 network
				manager & it's drivers
*/

#ifndef _NET_DRVR_H_INCLUDED
#define _NET_DRVR_H_INCLUDED

/************************************************************** DEFINES *****/

#define		VSP_INIT_DUP	 0x8000		/* possible dup of first vc seq num */

#define		QNX_CTRL_PKT		1		/* classes of qnx packets */
#define		QNX_DATA_PKT		2


#define		_HWT_ARCNET			0x01	/* physical media					*/
#define		_HWST_CORMAN		0x01	/* boot pkts start with x10 x00		*/
#define		_HWST_SMC			0x02	/* boot pkts start with xDC x66		*/
#define		_HWST_XIRCOM		0x03	/* xircom pocket arcnet adapter II	*/
#define		_HWST_20020 		0x04	/* smc com20020                   	*/

#define		_HWT_ETHERNET		0x02	/* boot pkts use 802.2 & ethertype	*/
#define		_HWST_WD8003		0x01	/* wd/smc 8003/8013 card			*/
#define		_HWST_NCR7052		0x02	/* NCR cash register Starlan (82588)*/
#define		_HWST_3COM503		0x03	/* 3com 503 etherlink II			*/
#define		_HWST_NE1000 		0x04	/* ne 1000 or 2000 		     		*/
#define		_HWST_3COM507		0x05	/* 3Com 507 ISA bus     			*/
#define		_HWST_3COM523		0x06	/* 3Com 523 MCA bus     			*/
#define		_HWST_FD       		0x07	/* driver uses fd for I/O			*/
#define		_HWST_79C960   		0x08	/* amd part - ne2100       			*/
#define		_HWST_NCR83593 		0x09	/* wavelan pcmcia          			*/
#define		_HWST_SMC9000  		0x0a	/* smc 91c90/91c92         			*/
#define		_HWST_DEC21X40 		0x0b	/* dec 21040/21140         			*/
#define		_HWST_ATT2MD_ETH	0x0c	/* at&t 100VG-AnyLAN, ethernet mode */
#define		_HWST_3COM509		0x0d	/* 3com 509 etherlink III			*/

#define		_HWST_ATT7231		0x0e	/* at&t 7231 (NCR 7450 Cash Register)*/
#define		_HWST_CS8900		0x0f	// crystal semiconductor 8900
#define		_HWST_3C905			0x10	// 3com 905 (boomerang)
#define		_HWT_TOKEN_RING		0x03	/* boot pkts use 802.2 & ethertype	*/
#define		_HWST_IBM164A		0x01	/* 8-bit ISA token-ring card		*/
#define		_HWST_ATT2MD_TR		0x02	/* at&t 100VG-AnyLAN, t-ring mode	*/

#define		_HWT_ZIATECH   		0x04	/* ziatech                        	*/
#define		_HWST_STAR  		0x01	/* multiple-cpu box     			*/

#define		_HWT_WAVEBUS   		0x05	/* AIT driver                     	*/

#define		_HWT_FDDI   		0x06	/* FDDI driver                     	*/
#define		_HWST_DFE   		0x01	/* Digital EISA FDDI controller		*/

#define		_HWT_ATM    		0x07	/* ATM drivers                     	*/
#define		_HWST_FORE200		0x01	/* FORE PCA-200PC ATM PCI card 		*/


typedef unsigned char IEEE_802_ADDR[6];


/************************************************************** STRUCTS *****/

struct _net_mgr {
	struct _net_q			 waiting_4_arp_q;	/* nid table lookup failed	*/
	struct _net_q			 txd_ok_q;			/* drivers put q pkts here	*/
	struct _net_q			 txd_bad_q;			/* drivers put q pkts here	*/

	short unsigned int		 tx_prep_seg;		/* ptr to tx munge function	*/
	long					 tx_prep_ofs;
	short unsigned int		 net_err_seg;		/* ptr to net error log fn	*/
	long					 net_err_ofs;

	short unsigned int		 msgr_pid;			/* net's proxy, drvr kicks	*/
	short unsigned int		 zero1;
	short unsigned int		 qpkt_seg;			/* GDT queue pkt data seg	*/
	short unsigned int		 zero2;
	short unsigned int		 data_seg;			/* net's data segment    	*/
	short unsigned int		 zero3;
	short unsigned int		 vid_mask;			/* convert vc's pid 2 index	*/
	short unsigned int		 zero4;

	short unsigned int		 vc_tx_seq_seg;		/* vid xmit sequence num	*/
	long					 vc_tx_seq_ofs;
	short unsigned int		 vc_rx_seq_seg;		/* vid rx   sequence num	*/
	long					 vc_rx_seq_ofs;

	long					 zero5;				/* pad to 80 bytes			*/
};

struct _net_drvr {
	short unsigned int		 pid;				/* driver's pid				*/
	short unsigned int		 zero1;
	short unsigned int		 lan_addr;			/* unique, assigned per lan	*/
	short unsigned int		 zero2;
	short unsigned int		 hw_type;			/* type of media			*/
	short unsigned int		 hw_subtype;		/* type of card 			*/
	short unsigned int		 flags;				/* information about driver	*/
	short unsigned int		 zero5;
	unsigned long			 media_rate;		/* driver hardware thruput	*/
	long					 zero4;
	IEEE_802_ADDR			 my_phys_nid;		/* this card's net address	*/
	short unsigned int		 zero6;

	short unsigned int		 sptr_seg;			/* drvr stk - aliased ss:sp	*/
	long					 sptr_ofs;			/*	for mgr far calls 2 drvr*/
	short unsigned int		 init_seg;			/* aliased far drvr fns		*/
	long					 init_ofs;			/*	called by net			*/
	short unsigned int		 stop_seg;
	long					 stop_ofs;
	short unsigned int		 tx_seg;			/* note all code segments	*/
	long					 tx_ofs;			/*  are assumed the same	*/
	short unsigned int		 rx_seg;
	long					 rx_ofs;
	short unsigned int		 rrcpy_seg;
	long					 rrcpy_ofs;

	unsigned char			 need_cpu_2_rx;		/* driver sets if pkts rxd	*/
	unsigned char			 need_cpu_2_tx;		/* driver sets if pkts txd	*/
	short unsigned int		 zero3;

	struct _net_q			 to_b_txd_q;		/* outgoing work q for drvr	*/
	struct _net_q			 txing_q;			/* xmissions in progress   	*/
};

/*
 *	struct located in driver space, describing rxd data (net gets ptr to)
 */
struct _pkt_info
{
	/*
	 *	hardware information about the packet
	 */
	IEEE_802_ADDR				phys_src_nid;
	IEEE_802_ADDR				phys_dst_nid;
	unsigned char				rpt[_RPT_LEN];

	/*
	 *	describe where the packet is located in driver data
	 */
	short unsigned int			buf_len;
	short unsigned int			buf_seg;
	long						buf_ofs;

	/*
	 *	flag which determines if qnx4 or "raw" packet
	 */
	short int					is_qnx4_vc_pkt;

	/*
	 *	qnx-only specific information
	 */
	unsigned char				class;
	unsigned char				type;
	short unsigned				vid;
	short unsigned				ext_vid;
	short unsigned				vid_seq;
};


/*
 *	msg (and reply) that net drivers sends to net manager to
 *	register as one of the network drivers
 */
struct _drvr2net_reg {
	short unsigned int		 type;
	short unsigned int		 subtype;
	struct _net_drvr		 info;
};
struct _drvr2net_reg_reply {
	short unsigned int		 status;
	short unsigned int		 zero1;
	short unsigned int		 zero2;
	short unsigned int		 zero3;
};

/*
 *	msg that net driver sends to net manager when driver
 *	want to shut down
 */
struct _drvr2net_dreg {
	short unsigned int		 type;
	short unsigned int		 subtype;
};
struct _drvr2net_dreg_reply {
	short unsigned int		 status;
	short unsigned int		 zero1;
};


/**************************************** FAR CALL FUNCTION DEFINES *********/

/*
 *	Note: these are for 16 bit drivers ONLY.  32 bit drivers will require
 *	different pragmas for far calling into Net (who is probably 16 bit)
 */

/*
 *	This is the mechanism used by a driver to far call up into the network
 *
 *	manager when the driver is about to transmit the contents pointed to
 *	by a proc->net queue packet.  The parameters are as follows:
 *
 *		dqp (in ax dx):	far pointer to driver's dummy queue packet, whose
 *						nlink MUST point to the real qpkt in the GDT
 *		buf (in bx cx):	far pointer to scratch buffer somewhere in driver
 *						data space.  buffer is at least as large as the
 *						largest packet supported by the driver media, plus
 *						room for an mx_entry at the start
 *		(n)->dataseg	short unsigned int - net's data segment	(on stk)
 *		(n)->tx_prep	far ptr to function in net_mgr			(on stk)
 *
 *	Simple drivers will have both the dummy queue packet & corresponding
 *	scratch buffer in their 64K data segment.  However, if a driver
 *	wishes to have (& manage) many scratch buffers, it may do so.
 */
#define TX_PREP(dqp,bf,n)	tp_fc(dqp,bf,(n)->data_seg,MK_FP((n)->tx_prep_seg,FP_OFF((n)->tx_prep_ofs)))
int tp_fc(struct _net_nq_pkt far *,char far *,short int,void far *);
#ifdef __386__
#pragma aux tp_fc =\
	"push ds"					\
	"mov  ds, 4[esp]"			\
	"call fword ptr 8[esp]"		\
	"pop  ds"					\
	"add  esp, 12"				\
parm [eax dx] [ebx cx]\
modify exact nomemory [eax ebx ecx edx];
#else
#pragma aux tp_fc =\
	0x55			/* push bp			; address parms on stk		*/	\
	0x8b 0xec		/* mov bp,sp									*/	\
	0x1e			/* push ds			; save drivers ds			*/	\
	0x8e 0x5e 0x02	/* mov ds,2[bp]		; ds = nets ds (on stack)	*/	\
	0xff 0x5e 0x04	/* abs call @4[bp] 	; far call func (on stk)	*/	\
	0x1f			/* pop ds			; restore regs				*/	\
	0x5d			/* pop bp										*/	\
	0x83 0xc4 0x06	/* add sp,#6		; correct stack for 2 args	*/	\
parm 	[ax dx] [bx cx] \
modify exact nomemory [ax bx cx dx];
#endif

/*
 *	This is the mechanism used by a driver to far call up into the network
 *	manager to log a network error. The parameters are as follows:
 *
 *		ne  (in ax dx):	far pointer to pre-stuffed struct _nerr_event
 *		    (in bx cx):	zero
 *		(n)->dataseg	short unsigned int - net's data segment	(on stk)
 *		(n)->net_err	far ptr to function in net_mgr			(on stk)
 *
 */
#define NET_ERR(ne,n)	ne_fc(ne,0L,(n)->data_seg,MK_FP((n)->net_err_seg,FP_OFF((n)->net_err_ofs)))
int ne_fc(struct _nerr_event far *, long, short int, void far *);
#ifdef __386__
#pragma aux ne_fc =\
	"push ds"					\
	"mov  ds, 4[esp]"			\
	"call fword ptr 8[esp]"		\
	"pop  ds"					\
	"add  esp, 12"				\
parm [eax dx] [ebx cx]\
modify exact nomemory [eax ebx ecx edx];
#else
#pragma aux ne_fc =\
	0x55			/* push bp			; address parms on stk		*/	\
	0x8b 0xec		/* mov bp,sp									*/	\
	0x1e			/* push ds			; save drivers ds			*/	\
	0x8e 0x5e 0x02	/* mov ds,2[bp]		; ds = nets ds (on stack)	*/	\
	0xff 0x5e 0x04	/* abs call @4[bp] 	; far call func (on stk)	*/	\
	0x1f			/* pop ds			; restore regs				*/	\
	0x5d			/* pop bp										*/	\
	0x83 0xc4 0x06	/* add sp,#6		; correct stack for 2 args	*/	\
parm 	[ax dx] [bx cx] \
modify exact nomemory [ax bx cx dx];
#endif

/****************************************************************************/
#endif

