/*
 *  net_nq.h    QNX 4.1 net enqueuing defines and struct definitions
 *
 *  Copyright (C) by Quantum Software Systems 1992.  All rights reserved.
 */
#ifndef __NET_NQ_H_INCLUDED

#ifndef __TYPES_H_INCLUDED
 #include <sys/types.h>
#endif

#pragma pack(1);

/************************************************************** DEFINES *****/

#define _VC_ATTACH          1   /*  types of things enqueued on net */
#define _VC_ATTACHED        2
#define _VC_DETACH          3
#define _VC_SEND            4
#define _VC_REPLY           5
#define _VC_RAW             6
#define _VC_PLS_GROW_VID    7   /*  send or reply faulted, grow on fly  */
#define _VC_PLS_GROW_LCL    8   /*  writemsg faulted, grow local buf only */
#define _VC_SIGNAL_PROP     9   /*  propogate signal across the net     */
#define _VC_POLL            10  /*  process manager generates           */
#define _VC_POLL_ACK        11  /*  positive response to POLL           */
#define _VC_POLL_NACK       12  /*  negative response to POLL           */
#define _VC_DRVR_ONLY       13  /*  driver enqueues on himself to tx    */
#define _VC_SIGNAL_BLK      14  /*  client signalled, unblk remote vid  */
#define _VC_FORCE_READY     15  /*  ready remote client blocked on vid  */
#define _VC_QNX2_VCNOTCRTD  16  /*  arcnet only - nack qnx2 vc_create   */
#define _VC_DRVR_ACK       100  /*  driver enqueues on himself to ack   */
#define _VC_DRVR_NACK      101  /*  driver enqueues on himself to nack  */
#define _VC_SPANS_ATM      110  /*  fore atm driver enqueues on himself */


#define _NUM_VID_PAIR_POLL  3


/*
 *	the following are used to make far pointers to the two  
 *	queue structs at the start of the proc->net q pkt gdt
 *	data segment.  Note that we simply reserve the first
 *	qpkt spot in the gdt - these ALL have to fit there.
 */
#define PROC_2_NET_Q_OFF	 (0 * sizeof(struct _net_q))
#define FREE_Q_OFF			 (1 * sizeof(struct _net_q))
#define	NUM_RESERVED_QPKTS	  1

#define IRQ_OFF				 1
#define IRQ_ON				 0


/************************************************************** STRUCTS *****/

												/* head & tail are the 		*/
												/* offset portions of the	*/
												/* far qpkts, which are all	*/
												/* in one segment - 2000 max*/
struct _net_q {
	short unsigned int		 head;				/* offset of ptr to head	*/
	short unsigned int		 tail;				/* offset of ptr to tail	*/
	short int				 len;				/* current length of this q */
	short int				 low;				/* telltale - min length	*/
	short int				 high;				/* telltale - max length	*/
	short int				 zero1;				/* pad to 12 bytes long		*/
};


/*
 *  The following is the format of the queue packet that either proc
 *  or net can get, stuff and enqueue onto net
 */

struct _net_nq_pkt {
    short unsigned           nlink;
    short unsigned           type;
    short unsigned           dst_nid;			/* was long */
    short unsigned           zero2;
    short unsigned           dst_vid,
                             ext_dst_vid;
    short unsigned           mx_seg;
#ifdef __386__
    long                     mx_off;
#else
    short unsigned           mx_off,
                             ext_mx_off;
#endif
    unsigned char            mx_parts;
    unsigned char            zero1;				/* was char vid_seq */
    short unsigned           remote_seg;
    short unsigned           remote_off,
                             ext_remote_off;	/* used for brg_nid */
    short unsigned           src_vid,
                             ext_src_vid;
	unsigned char			 num_tries;			/* used by ethernet & other */
	unsigned char			 num_ticks;			/* drvrs for unreliable media*/
    unsigned char            phys_dst_nid[6];
    short unsigned	         vid_seq;
    };	/* 40 bytes */

struct _vid_pair {
    short unsigned           src_vid,
                             ext_src_vid,
                             dst_vid,
                             ext_dst_vid;
    };  /*  8 bytes */

struct _net_nq_poll_pkt {
    short unsigned           nlink;
    short unsigned           type;
    short unsigned           dst_nid;
    short unsigned           vid_bits;
    struct _vid_pair         vid_pair[_NUM_VID_PAIR_POLL];
    unsigned char            phys_dst_nid[6];
    short unsigned	         zero_4_brg;  /* brg_nid, lines up with vid_seq */
    };  /* 40 bytes */


/************************************************* FUNCTION PROTOTYPES *****/

struct _net_nq_pkt far *q_get_first(struct _net_q far *, short int, short unsigned int );
struct _net_nq_pkt far *q_get_last (struct _net_q far *, short int, short unsigned int );

int q_put_first(struct _net_q far *, struct _net_nq_pkt far *, short int, short unsigned int );
int q_put_last (struct _net_q far *, struct _net_nq_pkt far *, short int, short unsigned int );

#pragma pack();

#define __NET_NQ_H_INCLUDED
#endif

